package net.serenitybdd.screenplay.webtests.model;

import com.google.common.base.Objects;

public class BankAccount
{
    private String name;

    private String bsb;

    private String accountNumber;

    public BankAccount( String name, String bsb, String accountNumber )
    {
        this.name = name;
        this.bsb = bsb;
        this.accountNumber = accountNumber;
    }

    public String getName()
    {
        return name;
    }

    public BankAccount setName( String name )
    {
        this.name = name;
        return this;
    }

    public String getBsb()
    {
        return bsb;
    }

    public String getAccountNumber()
    {
        return accountNumber;
    }


    @Override
    public boolean equals( Object o )
    {
        if ( this == o )
        {
            return true;
        }
        if ( o == null || getClass() != o.getClass() )
        {
            return false;
        }

        BankAccount that = ( BankAccount ) o;

        if ( name != null && that.name != null )
        {
            if ( name != null ? !name.equals( that.name ) : that.name != null )
            {
                return false;
            }
        }
        if ( accountNumber != null && that.accountNumber != null )
        {
            if ( accountNumber != null ? !accountNumber.equals( that.accountNumber ) : that.accountNumber != null )
            {
                return false;
            }
        }
        if ( bsb != null && that.bsb != null )
        {
            if ( bsb != null ? !bsb.equals( that.bsb ) : that.bsb != null )
            {
                return false;
            }
        }
        return true;
    }

    @Override
    public int hashCode()
    {
        int result = name != null ? name.hashCode() : 0;
        result = 31 * result + ( accountNumber != null ? accountNumber.hashCode() : 0 );
        result = 31 * result + ( bsb != null ? bsb.hashCode() : 0 );
        return result;
    }

    @Override
    public String toString()
    {
        return Objects.toStringHelper( this )
                .add( "accountName", name )
                .add( "accountNumber", accountNumber )
                .add( "bsb", bsb )
                .toString();
    }

    public static BankAccount forAStandardAccount()
    {
        return aStandardBankAccount();
    }

    public BankAccount withName( String newName )
    {
        return new BankAccount( newName, bsb, accountNumber );
    }

    public BankAccount withABsb(String bsb )
    {
        this.bsb = bsb;
        return this;
    }

    public BankAccount withAccountNumber(String accountNumber )
    {
        this.accountNumber = accountNumber;
        return this;
    }

    public BankAccount withAccountName(String accountName )
    {
        this.name = accountName;
        return this;
    }

    public static BankAccount aStandardBankAccount()
    {
        return new BankAccount(
                "Joe Blogs",
                "123456",
                "012345678");
    }

    public static BankAccount aBankAccount()
    {
        return new BankAccount(null, null, null);
    }

    public boolean matches(BankAccount that) {

        if ( name != null )
        {
            if ( name != null ? !name.equals( that.name ) : that.name != null )
            {
                return false;
            }
        }
        if ( accountNumber != null )
        {
            if ( accountNumber != null ? !accountNumber.equals( that.accountNumber ) : that.accountNumber != null )
            {
                return false;
            }
        }
        if ( bsb != null )
        {
            if ( bsb != null ? !bsb.equals( that.bsb ) : that.bsb != null )
            {
                return false;
            }
        }
        return true;
    }
}
